	

    #include <Ethernet.h>
    #include <SPI.h>
    #include "E131Constants.h"
    #include <FastLED.h>
     
    // Uncomment to turn on debug (will make it VERY SLOW)
    // The higher the numbr the more debug info (1-4)
//    #define E131_DEBUG 1
     
     
    // Each ethernet device needs its own MAC address
    // You can get your own mac string for this at
    //http://www.miniwebtool.com/mac-address-generator/
    byte mac[] = {0xDE, 0xAD, 0xEE, 0xFF, 0xDD, 0xED };
     
    //Change this IP to one on your network
    IPAddress ip(192, 168, 1, 30);
     
    // This is the starting universe, each string will take 1
    // String 1 == UNIVERSE
    // String 2 == UNIVERSE + 1
    // ........
    #define UNIVERSE 1
     
    ///////////////////////////////////////
    // This ones a bit harder to configure
    // For each string you must create 4-5 defines
    // They must be setup in order Eg 1 then 2.
    ////////////////////////////////////////////
     
    // Strip 1
    #define STRIP_PIN_1          3      // Strip 1 is on Pin 3
    #define STRIP_PIXELS_1       19     // Strip 1  Has 50 Pixels
    #define STRIP_CHIP_1         WS2801 // Strip 1 is using WS2811
    #define STRIP_RGB_1          GRB    // Strip 1 is using Red Green Blue order
    #define STRIP_CLOCK_1        4    // WS2801 needs clock so define clock pin
     
    // Strip 2
    #define STRIP_PIN_2          2      // Strip 2 is on Pin 2
    #define STRIP_PIXELS_2       15     // Strip 2  Has 50 Pixels
    #define STRIP_CHIP_2         WS2811 // Strip 2 is using WS2801  
    #define STRIP_RGB_2          BRG    // Strip 2 is using Green Red Blue order
//    #define STRIP_CLOCK_2        4       // WS2811 Does not use clock so do not define.
     
    // Strip 3
    //#define STRIP_PIN_3          5      // Strip 3 is on Pin 5
    //#define STRIP_PIXELS_3       50     // Strip 3  Has 50 Pixels
    //#define STRIP_CHIP_3         WS2811 // Strip 3 is using WS2811  
    //#define STRIP_RGB_3          RGB    // Strip 3 is using RGB order
    //#define STRIP_CLOCK_3        0      // WS2811 Does not use clock so do not define.
     
    // Strip 4
    //#define STRIP_PIN_4          6      // Strip 4 is on Pin 6
    //#define STRIP_PIXELS_4       50     // Strip 4  Has 50 Pixels
    //#define STRIP_CHIP_4         WS2811 // Strip 4 is using WS2811  
    //#define STRIP_RGB_4          RGB    // Strip 4 is using RGB order
    //#define STRIP_CLOCK_4        0      // WS2811 Does not use clock so do not define.
     
     
     
     
     
     
     
    ///////////////////////////////////////////////////////
    //    Should not have to edit anything below this    //
    ///////////////////////////////////////////////////////

 
// Workout the strip count and setup a pointer array for it
#if defined(STRIP_PIN_4) && defined(STRIP_PIXELS_4) && defined(STRIP_CHIP_4) && defined(STRIP_RGB_4)
  #define STRIP_COUNT 4
#elif defined(STRIP_PIN_3) && defined(STRIP_PIXELS_3) && defined(STRIP_CHIP_3) && defined(STRIP_RGB_3)
  #define STRIP_COUNT 3
#elif defined(STRIP_PIN_2) && defined(STRIP_PIXELS_2) && defined(STRIP_CHIP_2) && defined(STRIP_RGB_2)
  #define STRIP_COUNT 2
#elif defined(STRIP_PIN_1) && defined(STRIP_PIXELS_1) && defined(STRIP_CHIP_1) && defined(STRIP_RGB_1)
  #define STRIP_COUNT 1
#endif
 
// Setup arrays to hold pixel data
CRGB *Leds[STRIP_COUNT];
byte Pixels[STRIP_COUNT];
 
// Create the pixel data arrays
#if STRIP_COUNT >= 1
  CRGB Strip1[STRIP_PIXELS_1];
#endif
#if STRIP_COUNT >= 2
  CRGB Strip2[STRIP_PIXELS_2];
#endif  
#if STRIP_COUNT >= 3
  CRGB Strip3[STRIP_PIXELS_3];
#endif
#if STRIP_COUNT == 4
  CRGB Strip4[STRIP_PIXELS_4];
#endif  
 
 
uint8_t buffer1[638]; //buffer to hold incoming packet,
uint8_t* buf;
EthernetUDP Udp;
 
void setup(){
 
    // Ensure the SD Card is disabled
    pinMode(4, OUTPUT);
    digitalWrite(4, HIGH);
   
    #if defined(E131_DEBUG)
    Serial.begin(115200);
    Serial.print("Arduino E1.31 Strating with: ");  
    Serial.print(STRIP_COUNT);
    Serial.println(" strips.");
    #endif
   
    // Setup the Strings
    #if STRIP_COUNT >= 1
      #if defined(STRIP_CLOCK_1)
      FastLED.addLeds<STRIP_CHIP_1, STRIP_PIN_1, STRIP_CLOCK_1, STRIP_RGB_1>(Strip1, STRIP_PIXELS_1);
      #else
      FastLED.addLeds<STRIP_CHIP_1, STRIP_PIN_1, STRIP_RGB_1>(Strip1, STRIP_PIXELS_1);
      #endif
      Leds[0] = Strip1;
      Pixels[0] = STRIP_PIXELS_1;
      #if defined(E131_DEBUG)
      Serial.print("Enabled Strip 1 on pin ");
      Serial.print(STRIP_PIN_1);
      Serial.print(" with ");
      Serial.print(STRIP_PIXELS_1);
      Serial.println(" pixels.");
      #endif      
    #endif
   
    #if STRIP_COUNT >= 2
      #if defined(STRIP_CLOCK_2)
      FastLED.addLeds<STRIP_CHIP_2, STRIP_PIN_2, STRIP_CLOCK_2, STRIP_RGB_2>(Strip2, STRIP_PIXELS_2);
      #else
      FastLED.addLeds<STRIP_CHIP_2, STRIP_PIN_2, STRIP_RGB_2>(Strip2, STRIP_PIXELS_2);
      #endif
      Leds[1] = Strip2;
      Pixels[1] = STRIP_PIXELS_2;
      #if defined(E131_DEBUG)
      Serial.print("Enabled Strip 2 on pin ");
      Serial.print(STRIP_PIN_2);
      Serial.print(" with ");
      Serial.print(STRIP_PIXELS_2);
      Serial.println(" pixels.");
      #endif      
    #endif  
   
    #if STRIP_COUNT >= 3
      #if defined(STRIP_CLOCK_3)
      FastLED.addLeds<STRIP_CHIP_3, STRIP_PIN_3, STRIP_CLOCK_3, STRIP_RGB_3>(Strip3, STRIP_PIXELS_3);
      #else
      FastLED.addLeds<STRIP_CHIP_3, STRIP_PIN_3, STRIP_RGB_3>(Strip3, STRIP_PIXELS_3);
      #endif
      Leds[2] = Strip3;
      Pixels[2] = STRIP_PIXELS_3;
      #if defined(E131_DEBUG)
      Serial.print("Enabled Strip 3 on pin ");
      Serial.print(STRIP_PIN_3);
      Serial.print(" with ");
      Serial.print(STRIP_PIXELS_3);
      Serial.println(" pixels.");
      #endif      
    #endif
   
    #if STRIP_COUNT == 4
      #if defined(STRIP_CLOCK_4)
      FastLED.addLeds<STRIP_CHIP_4, STRIP_PIN_4, STRIP_CLOCK_4, STRIP_RGB_4>(Strip4, STRIP_PIXELS_4);
      #else
      FastLED.addLeds<STRIP_CHIP_4, STRIP_PIN_4, STRIP_RGB_4>(Strip4, STRIP_PIXELS_4);
      #if
      Leds[3] = Strip4;
      Pixels[2] = STRIP_PIXELS_4;
      #endif defined(E131_DEBUG)
      Serial.print("Enabled Strip 4 on pin ");
      Serial.print(STRIP_PIN_4);
      Serial.print(" with ");
      Serial.print(STRIP_PIXELS_4);
      Serial.println(" pixels.");
      #endif      
    #endif  
   
               
    // Setup the UDP socket to listen
    uint16_t PortNr = 5568;
    Ethernet.begin(mac,ip);
    Udp.begin(PortNr);
 
    #if defined(E131_DEBUG)
    Serial.print("Free Memory: ");
    Serial.println(availableMemory());
    #endif
   
}
 
 
 
#if defined(E131_DEBUG)
int availableMemory() {
  int size = 2048; // Use 2048 with ATmega328
  byte *buf;
 
  while ((buf = (byte *) malloc(--size)) == NULL)
    ;
  free(buf);
  return size;
}
#endif
 
 
 
 
 
uint16_t numChannelsInPacket;
uint16_t universe;
void loop () {
    // Read new E1.31 Data    
    if(Udp.parsePacket() > 0) {
       #if E131_DEBUG > 1
          Serial.println("Received UDP Packet");
       #endif    
        buf=buffer1;    
        // Read data from udp socket
        Udp.read(buf, 638);            
        if(validateRoot(buf) && validateDMP(buf) && validateFraming(buf)) {            
            numChannelsInPacket = ( (buf[E1_31_DMP_PROP_VAL_CNT] <<8)  | buf[E1_31_DMP_PROP_VAL_CNT+1] ) -1 ;
            universe = ((buf[E1_31_FRAMING_UNIVERSE_ID] <<8) | buf[E1_31_FRAMING_UNIVERSE_ID+1]);          
            if (numChannelsInPacket <=512 && numChannelsInPacket > 0 && universe >= UNIVERSE && universe <= (UNIVERSE + (STRIP_COUNT-1))) {                              
                // Write the new data to the light strip.
                uint16_t id = universe - UNIVERSE;
                int Nodes = Pixels[id];                
                uint16_t n, LastChan = E1_31_DMP_FIRST+numChannelsInPacket;
               
                #if E131_DEBUG > 2
                  Serial.print("Updating strip ");  
                  Serial.print(id);
                  Serial.print(" with data from universe ");
                  Serial.println(universe);
                #endif                  
               
                for(uint16_t i=E1_31_DMP_FIRST, n=0; i < LastChan && n < Nodes; n++) {
                    #if E131_DEBUG > 3
                      Serial.print("Setting Pixel ");  
                      Serial.print(n);
                      Serial.print(" R:");
                      Serial.print(buf[i]);
                      Serial.print(" G:");
                      Serial.print(buf[i+1]);
                      Serial.print(" B:");
                      Serial.println(buf[i+2]);
                    #endif  
                 
                    // Update the node
                    CRGB *Strip = Leds[id];
                    Strip[n].r = buf[i++];
                    Strip[n].g = buf[i++];
                    Strip[n].b = buf[i++];            
                }
                // Send data to the strip
                FastLED.show();
            }
            #if E131_DEBUG > 2
            else {                  
                   Serial.print("Ignoring packet for universe: ");
                   Serial.println(universe);                    
            }
            #endif            
        }
    }        
}
 
 
 
bool validateDMP(uint8_t buf[]) {
        if (buf[E1_31_DMP_VECTOR] != E1_31_DMP_VECTOR_VALID
        || buf[E1_31_DMP_ADDRESS_TYPE_DATA_TYPE] != E1_31_DMP_ADDRESS_TYPE_DATA_TYPE_VALID     
        || ntos(buf + E1_31_DMP_ADDRESS_INCREMENT) != E1_31_DMP_ADDRESS_INCREMENT_VALID)
        return false;
        //         ntos(buf + E1_31_DMP_PROP_VAL_CNT) - 1;
        return true;
//|| ntos(buf + E1_31_DMP_FIRST_PROP_ADDR) != E1_31_DMP_FIRST_PROP_ADDR_VALID
}
 
bool validateFraming (uint8_t buf[]) {
        if (ntoi(buf + E1_31_FRAMING_VECTOR) != E1_31_FRAMING_VECTOR_VALID)
        return false;
        return true;
}
 
bool validateRoot (uint8_t buf[]) {
 
        if (ntos(buf) != RLP_PREAMBLE_SIZE_VALID
        || ntos(buf + RLP_POST_AMBLE_SIZE) != RLP_POST_AMBLE_SIZE_VALID
        || ntoi(buf + IDX_VECTOR_PROTOCOL) != 0x00000004)
        return false;
        return true;
}
 
 
static uint32_t ntoi(uint8_t *buf) {
    return static_cast<uint32_t>(buf[2]) << 24
    | static_cast<uint32_t>(buf[1]) << 16
    | static_cast<uint32_t>(buf[0]) << 8
    | buf[3];
}
 
static uint16_t ntos(uint8_t *buf) {
    return static_cast<uint16_t>(buf[0]) << 8 | buf[1];
}
